@@ -41,8 +41,8 @@
 
 /*===========================================================================*\
  *                                                                           *             
- *   $Revision: 1258 $                                                         *
- *   $Date: 2015-04-28 15:07:46 +0200 (Di, 28 Apr 2015) $                   *
+ *   $Revision$                                                         *
+ *   $Date$                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -89,7 +89,7 @@ public:
 public:
 
    /// Constructor
-  ModBalancerT( D &_dec )
+  explicit ModBalancerT( D &_dec )
     : BaseModQ( _dec ),
       max_level_(0), n_roots_(0), n_vertices_(0)
   {
