@@ -1,41 +1,48 @@
-/*===========================================================================*\
+/* ========================================================================= *
  *                                                                           *
  *                               OpenMesh                                    *
- *      Copyright (C) 2001-2015 by Computer Graphics Group, RWTH Aachen      *
- *                           www.openmesh.org                                *
+ *           Copyright (c) 2001-2015, RWTH-Aachen University                 *
+ *           Department of Computer Graphics and Multimedia                  *
+ *                          All rights reserved.                             *
+ *                            www.openmesh.org                               *
  *                                                                           *
  *---------------------------------------------------------------------------*
- *  This file is part of OpenMesh.                                           *
+ * This file is part of OpenMesh.                                            *
+ *---------------------------------------------------------------------------*
  *                                                                           *
- *  OpenMesh is free software: you can redistribute it and/or modify         *
- *  it under the terms of the GNU Lesser General Public License as           *
- *  published by the Free Software Foundation, either version 3 of           *
- *  the License, or (at your option) any later version with the              *
- *  following exceptions:                                                    *
+ * Redistribution and use in source and binary forms, with or without        *
+ * modification, are permitted provided that the following conditions        *
+ * are met:                                                                  *
  *                                                                           *
- *  If other files instantiate templates or use macros                       *
- *  or inline functions from this file, or you compile this file and         *
- *  link it with other files to produce an executable, this file does        *
- *  not by itself cause the resulting executable to be covered by the        *
- *  GNU Lesser General Public License. This exception does not however       *
- *  invalidate any other reasons why the executable file might be            *
- *  covered by the GNU Lesser General Public License.                        *
+ * 1. Redistributions of source code must retain the above copyright notice, *
+ *    this list of conditions and the following disclaimer.                  *
  *                                                                           *
- *  OpenMesh is distributed in the hope that it will be useful,              *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
- *  GNU Lesser General Public License for more details.                      *
+ * 2. Redistributions in binary form must reproduce the above copyright      *
+ *    notice, this list of conditions and the following disclaimer in the    *
+ *    documentation and/or other materials provided with the distribution.   *
  *                                                                           *
- *  You should have received a copy of the GNU LesserGeneral Public          *
- *  License along with OpenMesh.  If not,                                    *
- *  see <http://www.gnu.org/licenses/>.                                      *
+ * 3. Neither the name of the copyright holder nor the names of its          *
+ *    contributors may be used to endorse or promote products derived from   *
+ *    this software without specific prior written permission.               *
  *                                                                           *
-\*===========================================================================*/
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED *
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           *
+ * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER *
+ * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  *
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       *
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        *
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    *
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      *
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        *
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              *
+ *                                                                           *
+ * ========================================================================= */
 
 /*===========================================================================*\
  *                                                                           *
- *   $Revision: 1188 $                                                         *
- *   $Date: 2015-01-05 16:34:10 +0100 (Mo, 05 Jan 2015) $                   *
+ *   $Revision$                                                         *
+ *   $Date$                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -48,11 +55,9 @@
 #include <limits>
 
 // OpenMesh
-#include <OpenMesh/Core/System/config.h>
 #include <OpenMesh/Core/IO/BinaryHelper.hh>
 #include <OpenMesh/Core/IO/writer/OBJWriter.hh>
 #include <OpenMesh/Core/IO/IOManager.hh>
-#include <OpenMesh/Core/System/omstream.hh>
 #include <OpenMesh/Core/Utils/color_cast.hh>
 
 //=== NAMESPACES ==============================================================
@@ -110,10 +115,10 @@ write(const std::string& _filename, BaseExporter& _be, Options _opt, std::stream
     }
 
     //remove the file extension
-    dot = _filename.find_last_of(".");
+    dot = objName_.find_last_of(".");
 
     if(dot != std::string::npos)
-      objName_ = objName_.substr(0,dot-1);
+      objName_ = objName_.substr(0,dot);
   }
 
   bool result = write(out, _be, _opt, _precision);
@@ -177,18 +182,18 @@ writeMaterial(std::ostream& _out, BaseExporter& _be, Options _opt) const
   //write the materials
   if ( _opt.color_has_alpha() )
     for (size_t i=0; i < materialA_.size(); i++){
-      _out << "newmtl " << "mat" << i << std::endl;
-      _out << "Ka 0.5000 0.5000 0.5000" << std::endl;
-      _out << "Kd " << materialA_[i][0] << materialA_[i][1] << materialA_[i][2] << std::endl;;
-      _out << "Tr " << materialA_[i][3] << std::endl;
-      _out << "illum 1" << std::endl;
+      _out << "newmtl " << "mat" << i << '\n';
+      _out << "Ka 0.5000 0.5000 0.5000" << '\n';
+      _out << "Kd " << materialA_[i][0] << ' ' << materialA_[i][1] << ' ' << materialA_[i][2] << '\n';
+      _out << "Tr " << materialA_[i][3] << '\n';
+      _out << "illum 1" << '\n';
     }
   else
     for (size_t i=0; i < material_.size(); i++){
-      _out << "newmtl " << "mat" << i << std::endl;
-      _out << "Ka 0.5000 0.5000 0.5000" << std::endl;
-      _out << "Kd " << material_[i][0] << material_[i][1] << material_[i][2] << std::endl;;
-      _out << "illum 1" << std::endl;
+      _out << "newmtl " << "mat" << i << '\n';
+      _out << "Ka 0.5000 0.5000 0.5000" << '\n';
+      _out << "Kd " << material_[i][0] << ' ' << material_[i][1] << ' ' << material_[i][2] << '\n';
+      _out << "illum 1" << '\n';
     }
 
   return true;
@@ -219,12 +224,22 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
   if (!check( _be, _opt))
      return false;
 
+  // No binary mode for OBJ
+  if ( _opt.check(Options::Binary) ) {
+    omout() << "[OBJWriter] : Warning, Binary mode requested for OBJ Writer (No support for Binary mode), falling back to standard." <<  std::endl;
+  }
 
-  // check writer features
-  if ( _opt.check(Options::Binary)     || // not supported by format
-       _opt.check(Options::FaceNormal))
-     return false;
+  // check for unsupported writer features
+  if (_opt.check(Options::FaceNormal) ) {
+    omerr() << "[OBJWriter] : FaceNormal not supported by OBJ Writer" <<  std::endl;
+    return false;
+  }
 
+  // check for unsupported writer features
+  if (_opt.check(Options::VertexColor) ) {
+    omerr() << "[OBJWriter] : VertexColor not supported by OBJ Writer" <<  std::endl;
+    return false;
+  }
 
   //create material file if needed
   if ( _opt.check(Options::FaceColor) ){
@@ -246,11 +261,47 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
 
   // header
   _out << "# " << _be.n_vertices() << " vertices, ";
-  _out << _be.n_faces() << " faces" << std::endl;
+  _out << _be.n_faces() << " faces" << '\n';
 
   // material file
   if (useMatrial &&  _opt.check(Options::FaceColor) )
-    _out << "mtllib " << objName_ << ".mat" << std::endl;
+    _out << "mtllib " << objName_ << ".mat" << '\n';
+
+  std::map<Vec2f,int> texMap;
+  //collect Texturevertices from halfedges
+  if(_opt.check(Options::FaceTexCoord))
+  {
+    std::vector<Vec2f> texCoords;
+    //add all texCoords to map
+    unsigned int num = _be.get_face_texcoords(texCoords);
+    for(unsigned int i = 0; i < num ; ++i)
+    {
+      texMap[texCoords[i]] = i;
+    }
+  }
+
+  //collect Texture coordinates from vertices
+  if(_opt.check(Options::VertexTexCoord))
+  {
+    for (size_t i=0, nV=_be.n_vertices(); i<nV; ++i)
+    {
+      vh = VertexHandle(static_cast<int>(i));
+      t  = _be.texcoord(vh);
+      texMap[t] = static_cast<int>(i);
+    }
+  }
+
+  // assign each texcoord in the map its id
+  // and write the vt entries
+  if(_opt.check(Options::VertexTexCoord) || _opt.check(Options::FaceTexCoord))
+  {
+    int texCount = 0;
+    for(std::map<Vec2f,int>::iterator it = texMap.begin(); it != texMap.end() ; ++it)
+    {
+      _out << "vt " << it->first[0] << " " << it->first[1] << '\n';
+      it->second = ++texCount;
+    }
+  }
 
   // vertex data (point, normals, texcoords)
   for (i=0, nV=_be.n_vertices(); i<nV; ++i)
@@ -260,20 +311,18 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
     n  = _be.normal(vh);
     t  = _be.texcoord(vh);
 
-    _out << "v " << v[0] <<" "<< v[1] <<" "<< v[2] << std::endl;
+    _out << "v " << v[0] <<" "<< v[1] <<" "<< v[2] << '\n';
 
     if (_opt.check(Options::VertexNormal))
-      _out << "vn " << n[0] <<" "<< n[1] <<" "<< n[2] << std::endl;
-
-    if (_opt.check(Options::VertexTexCoord))
-      _out << "vt " << t[0] <<" "<< t[1] << std::endl;
+      _out << "vn " << n[0] <<" "<< n[1] <<" "<< n[2] << '\n';    
   }
 
   size_t lastMat = std::numeric_limits<std::size_t>::max();
 
   // we do not want to write seperators if we only write vertex indices
   bool onlyVertices =    !_opt.check(Options::VertexTexCoord)
-                      && !_opt.check(Options::VertexNormal);
+                      && !_opt.check(Options::VertexNormal)
+                      && !_opt.check(Options::FaceTexCoord);
 
   // faces (indices starting at 1 not 0)
   for (i=0, nF=_be.n_faces(); i<nF; ++i)
@@ -294,7 +343,7 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
 
       // if we are ina a new material block, specify in the file which material to use
       if(lastMat != material) {
-        _out << "usemtl mat" << material << std::endl;
+        _out << "usemtl mat" << material << '\n';
         lastMat = material;
       }
     }
@@ -314,9 +363,18 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
         // write separator
         _out << "/" ;
 
-        // write vertex texture coordinate index
-        if (_opt.check(Options::VertexTexCoord))
-          _out  << idx;
+        //write texCoords index from halfedge
+        if(_opt.check(Options::FaceTexCoord))
+        {
+          _out << texMap[_be.texcoord(_be.getHeh(FaceHandle(int(i)),vhandles[j]))];
+        }
+
+        else
+        {
+          // write vertex texture coordinate index
+          if (_opt.check(Options::VertexTexCoord))
+            _out  << texMap[_be.texcoord(vhandles[j])];
+        }
 
         // write vertex normal index
         if ( _opt.check(Options::VertexNormal) ) {
@@ -327,7 +385,7 @@ write(std::ostream& _out, BaseExporter& _be, Options _opt, std::streamsize _prec
       }
     }
 
-    _out << std::endl;
+    _out << '\n';
   }
 
   material_.clear();
